"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FormMethods = void 0;
var index_1 = __importDefault(require("lodash.isobject/index"));
var Errors_1 = __importDefault(require("./Errors"));
var Request_1 = __importDefault(require("./Request"));
var FormMethods;
(function (FormMethods) {
    FormMethods["POST"] = "post";
    FormMethods["PATCH"] = "patch";
    FormMethods["PUT"] = "put";
})(FormMethods = exports.FormMethods || (exports.FormMethods = {}));
var Form = /** @class */ (function () {
    /**
     * Create a new Form instance.
     */
    function Form(data, config) {
        var _this = this;
        if (config === void 0) { config = {}; }
        var _a;
        this.touchedFields = {};
        this.config = __assign({ resetAfterSend: true, removeNullValues: true, url: null, method: FormMethods.POST }, config);
        Object.keys(data).forEach(function (key) {
            _this[key] = data[key];
        });
        this.originalData = __assign({}, data);
        this.errors = new Errors_1.default();
        this.request = (_a = this.config.request) !== null && _a !== void 0 ? _a : Request_1.default.getInstance();
        if (this.config.method !== FormMethods.POST) {
            this.addField('_method', this.config.method);
        }
    }
    /**
     * Add a new field with value to the form.
     */
    Form.prototype.addField = function (field, value) {
        this.originalData[field] = value;
        this[field] = value;
    };
    Form.prototype.setUrl = function (url) {
        this.config.url = url;
        return this;
    };
    Form.prototype.addTouchedField = function (field) {
        this.touchedFields[field] = field;
    };
    Form.prototype.getTouchedFields = function () {
        return Object.values(this.touchedFields);
    };
    /**
     * Serialize the Form.
     */
    Form.prototype.serialize = function (asString) {
        var _this = this;
        if (asString === void 0) { asString = true; }
        var json = {};
        Object.keys(this.originalData).forEach(function (field) {
            json[field] = _this[field];
        });
        return asString ? JSON.stringify(json) : json;
    };
    /**
     * Submit the form.
     */
    Form.prototype.submit = function (url) {
        this.errors.clear();
        var data = this.getFormData();
        return this.send(this.getUrl(url), data);
    };
    Form.prototype.validate = function (url) {
        var data = this.getFormData();
        data.append('validate', '1');
        this.getTouchedFields().forEach(function (field) {
            data.append('fields[]', field);
        });
        return this.send(this.getUrl(url), data, true);
    };
    /**
     * Get FormData object.
     */
    Form.prototype.getFormData = function () {
        var _this = this;
        var formData = new FormData();
        Object.keys(this.originalData).forEach(function (field) {
            var value = _this[field];
            if (value === null && _this.config.removeNullValues) {
                return;
            }
            if (typeof value === 'string'
                && value.length === 0
                && _this.config.removeNullValues) {
                return;
            }
            if (typeof value === 'boolean') {
                formData.append(field, Number(value).toString());
                return;
            }
            if (Array.isArray(value)) {
                _this.addArray(field, value, formData);
                return;
            }
            if (index_1.default(value) && !(value instanceof File)) {
                _this.addObject(field, value, formData);
                return;
            }
            formData.append(field, value);
        });
        return formData;
    };
    /**
     * Reset the state of the form to the original state.
     */
    Form.prototype.reset = function () {
        var _this = this;
        Object.keys(this.originalData).forEach(function (field) {
            _this[field] = _this.originalData[field];
        });
    };
    Form.prototype.addArray = function (field, array, formData) {
        if (array.length === 0 && this.config.removeNullValues) {
            return;
        }
        array.forEach(function (item) {
            formData.append(field + "[]", item);
        });
    };
    Form.prototype.addObject = function (field, object, formData) {
        var keys = Object.keys(object);
        if (keys.length === 0 && this.config.removeNullValues) {
            return;
        }
        keys.forEach(function (key) {
            formData.append(field + "[" + key + "]", object[key]);
        });
    };
    Form.prototype.getUrl = function (url) {
        if (url) {
            return url;
        }
        if (this.config.url) {
            return this.config.url;
        }
        throw new Error('Please specify url.');
    };
    Form.prototype.send = function (url, data, shouldValidate) {
        var _this = this;
        if (shouldValidate === void 0) { shouldValidate = false; }
        var config = { headers: { 'Content-Type': 'multipart/form-data' } };
        return new Promise(function (resolve, reject) {
            _this.request.post(url, data, config)
                .then(function (response) {
                if (_this.config.resetAfterSend && !shouldValidate) {
                    _this.reset();
                }
                _this.errors.clear();
                return resolve(response.data);
            })
                .catch(function (error) {
                var _a, _b;
                if (typeof ((_a = error.response) === null || _a === void 0 ? void 0 : _a.data.errors) !== 'undefined') {
                    _this.errors.record(error.response.data.errors);
                }
                return reject((_b = error.response) === null || _b === void 0 ? void 0 : _b.data);
            });
        });
    };
    return Form;
}());
exports.default = Form;
//# sourceMappingURL=Form.js.map